local mod = ...


-- Assets -------------------------------------------------------------------------------------
mod:registerAssetId("models/OrchardFarm1.fbx/Prefab/FarmPart", "PREFAB_ORCHARD_FARM1")
mod:registerAssetId("models/OrchardFarm1.fbx/Prefab/CartPart", "PREFAB_ORCHARD_CART")
mod:registerAssetId("models/TreeApple3.fbx/Prefab/Tree", "PREFAB_ORCHARD_APPLE1")
-- mod:registerAssetId("models/Textures/AppleTree.png", "TEXTURE_ORCHARD_APPLE")



-- Construction visual
mod:registerAssetId("models/OrchardFarm1.fbx/Prefab/ConstructionSteps", "PREFAB_CONSTR_ORCHARD_FARM1")



-- Grounding  -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/OrchardFarm1.fbx/Prefab/FarmPart/Ladder", {DataType = "COMP_GROUNDED"})

for i=1, 6, 1 do
	mod:registerPrefabComponent("models/OrchardFarm1.fbx/Prefab/FarmPart/Barrel." .. string.sub("000" .. i, -3), {DataType = "COMP_GROUNDED"})
end



-- Tree Collision Zone  -------------------------------------------------------------------------------------
-- This code causes a crash on re-loading the game
-- mod:registerPrefabComponent("models/TreeApple3.fbx/Prefab/Tree", {
--     DataType = "COMP_BUILDING_ZONE", 
--     BuildingZone = {
--         ZoneEntryList = {
--             { 
--                 Polygon = polygon.createCircle( 1, {0.0, 0.0}, 6),
--                 Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = false } 
--             },
--         }
--     }
-- })



-- Paths: Apple Cart
mod:registerAssetId("models/OrchardFarm1.fbx/Prefab/CartPart/PATH_M_1", "PATH_ORCHARD_CART1")
mod:registerAssetId("models/OrchardFarm1.fbx/Prefab/CartPart/PATH_N_1", "PATH_ORCHARD_CART2")
mod:registerAssetId("models/OrchardFarm1.fbx/Prefab/CartPart/PATH_O_1", "PATH_ORCHARD_CART3")
mod:registerAssetId("models/OrchardFarm1.fbx/Prefab/CartPart/PATH_P_1", "PATH_ORCHARD_CART4")

mod:registerPrefabComponent("models/OrchardFarm1.fbx/Prefab/CartPart", {
    DataType = "COMP_BUILDING_PART",
    PathList = {
        {
            PathType = BUILDING_PATH_TYPE.DEFAULT,
            WayPointList = { "PATH_ORCHARD_CART1" }
        },
        {
            PathType = BUILDING_PATH_TYPE.PICKUP,
            WayPointList = { "PATH_ORCHARD_CART2" }
        },
        {
            PathType = BUILDING_PATH_TYPE.PICKUP,
            WayPointList = { "PATH_ORCHARD_CART3" }
        },
        {
            PathType = BUILDING_PATH_TYPE.PICKUP,
            WayPointList = { "PATH_ORCHARD_CART4" }
        },
    }
})



-- Resource Container: Tree  -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/TreeApple3.fbx/Prefab/Tree", {
    DataType = "COMP_RESOURCE_CONTAINER",
    ResourceData = "RESOURCE_FRUIT",
    IsReplenishable = true,
    ReplenishDurationInDays = 7,
    ReplenishQuantity = 1,
    IsDestroyWhenEmpty = false, 
    AvailableQuantity = 1, -- initial quantity per tree
    ResourceValue = 1, -- amount in each picking action
    Radius = 1.5 -- distance from the container to stand
})



-- Plantable -------------------------------------------------------------------------------------
-- This model must have only a single Mesh and a single Material!
mod:registerPrefabComponent("models/TreeApple3.fbx/Prefab/Tree", { 
    DataType = "COMP_PLANTABLE", 
    MinSpaceBetweenPlant = 15.0 
})

mod:register({
    DataType = "PLANTABLE",
    Id = "PLANTABLE_APPLE1",
    Name = "PLANTABLE_APPLE1_NAME",
    PlantPrefab = "PREFAB_ORCHARD_APPLE1",
    MinimumScale = 0.95,
    MaximumScale = 1.05
})



-- Crop Field Element -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/TreeApple3.fbx/Prefab/Tree", { DataType = "COMP_CROP_FIELD_ELEMENT" })



-- Alpha material (transparency) -------------------------------------------------------------------------------------
mod:registerAssetId("models/TreeApple3.fbx/Materials/Material.Tree", "MATERIAL_ORCHARD_APPLE")

mod:override({
    Id = "MATERIAL_ORCHARD_APPLE",
    HasAlphaTest = true
})



-- Building Asset Processor  -------------------------------------------------------------------------------------
mod:registerAssetProcessor("models/OrchardFarm1.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })



-- Zone -------------------------------------------------------------------------------------
mod:register({
    DataType = "ZONE",
    Id = "ZONE_ORCHARD_FARM",
    Name = "ZONE_ORCHARD_FARM_NAME",
    Description = "ZONE_ORCHARD_FARM_DESC",
    Color = {2.20, 2.50, 1.70, 0.9}, -- RGB A = Light green
    DesirabilityLayerToShow = "DESIRABILITY_FERTILITY"
})



-- Farm Field Config -------------------------------------------------------------------------------------

mod:register({
    DataType = "FARM_FIELD_CONFIG",
    Id = "FARM_FIELD_CONFIG_ORCHARD_FARM_APPLE",
    ZoneType = "ZONE_ORCHARD_FARM",
    PlantPrefab = "PREFAB_ORCHARD_APPLE1",
    HasPlantingCycle = false,
    IsScaling = false,
    RowDistance = 5.25,
    ItemDistance = 5.25,
    RandomOrientation = {0, 360},
    RandomPositionOffset = {-.5, .5},
    SourceMaterial = "MATERIAL_ORCHARD_APPLE",
    GrowingMaterial = "MATERIAL_ORCHARD_APPLE",
    HarvestedMaterial = "MATERIAL_ORCHARD_APPLE",
    GrowingTextureList = {
        -- "TEXTURE_ORCHARD_APPLE",
        -- "TEXTURE_TREE_POPLAR",
    },
    HarvestingTextureList = {
        -- "TEXTURE_ORCHARD_APPLE",
        -- "TEXTURE_TREE_POPLAR",
    },
    AlphaTestThreshold = 0.5
})



-- Growing Setup -------------------------------------------------------------------------------------

mod:register({
    DataType = "AGENT_WORKSTATION_SETUP",
    Id = "AGENT_WORKSTATION_SETUP_ORCHARD_FARM_GROWING",
	CharacterSetup = {
        DataType = "CHARACTER_SETUP",
		-- RightHandObject = {
		-- 	DataType = "HAND_OBJECT",
		-- 	Model = "PREFAB_TOOL_SHEPHERD_STICK"
		-- },
		IdleAnimation = "MARKET_TENDER"
	}
})



-- Sowing Setup -------------------------------------------------------------------------------------

mod:register({
    DataType = "AGENT_WORKSTATION_SETUP",
    Id = "AGENT_WORKSTATION_SETUP_ORCHARD_FARM_SOWING",
	CharacterSetup = {
        DataType = "CHARACTER_SETUP",
		-- RightHandObject = {
		-- 	DataType = "HAND_OBJECT",
		-- 	Model = "PREFAB_TOOL_SHEPHERD_STICK"
		-- },
		IdleAnimation = "MARKET_TENDER"
	}
})



-- Harvest Setup -------------------------------------------------------------------------------------

mod:register({
    DataType = "AGENT_WORKSTATION_SETUP",
    Id = "AGENT_WORKSTATION_SETUP_ORCHARD_FARM_HARVEST",
	CharacterSetup = {
        DataType = "CHARACTER_SETUP",
		-- RightHandObject = {
		-- 	DataType = "HAND_OBJECT",
		-- 	Model = "PREFAB_TOOL_SHEPHERD_STICK"
		-- },
		IdleAnimation = "MARKET_TENDER"
	}
})




-- Resource -------------------------------------------------------------------------------------
-- Moved to Common Resources mod
-- mod:registerAssetId("icons/Fruit.png", "ICON_FRUIT")

-- mod:register({
-- 	DataType = "RESOURCE",
-- 	Id = "RESOURCE_FRUIT",
-- 	ResourceName = "RESOURCE_FRUIT_NAME",
-- 	Icon = "ICON_FRUIT",
-- 	ResourceTypeList = { "FOOD" },
-- 	IsTradable = true,
-- 	ResourceLayoutType = "CRATES",
-- 	VillagerBuyingPrice = { Resource = "GOLD", Quantity = 10 },
-- 	DisplayInInventory = true
-- })


-- Job -------------------------------------------------------------------------------------

mod:register({
	DataType = "JOB",
	Id = "JOB_ORCHARD",
	JobName = "JOB_ORCHARD_NAME",
	JobDescription = "JOB_ORCHARD_DESC",
	IsLockedByDefault = false,
	DefaultBehavior = "BEHAVIOR_FARMER",
    ProductionDelay = 5.0, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP",
        -- FemaleHatList = {
        --     "PREFAB_HAT_FARMER3",
        -- }, 
        -- MaleHatList = {
        --     "PREFAB_HAT_FARMER3",
        -- }, 
		WalkAnimation = "WALKING",
		IdleAnimation = "MARKET_TENDER"
	}
})

mod:override({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_ORCHARD"
    }
})

mod:override({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_ORCHARD"
    }
})

mod:override({
    Id = "COMMONER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_ORCHARD"
    }
})



-- Building Function: Orchard -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_FARM",
    Id = "BUILDING_FUNCTION_ORCHARD",
    Name = "BUILDING_FUNCTION_ORCHARD_NAME",
    WorkerCapacity = 3,
    RelatedJob = { 
        Job = "JOB_ORCHARD", 
        Behavior = "BEHAVIOR_FARMER"
    },
    -- InputInventoryCapacity = {
		-- { Resource = "WATER", Quantity = 5 },
		-- { Resource = "RESOURCE_DUNG", Quantity = 5 }
	-- },
    ResourceListNeeded = {
		{ Resource = "RESOURCE_FRUIT", Quantity = 1 }, 
	},
	ResourceProduced = {
        { Resource = "RESOURCE_FRUIT", Quantity = 1 },
    },

    -- there is a max total cycle of 21 days so if you plan beyond that, it will be cut short
    SowingDurationPerc = {0, 1}, 
	GrowDuration = {1, 2}, 
	GrowWaitDuration = {2, 6},
	HarvestDuration = {6, 18},
	RestDuration = {18, 21},

    QuantityToHarvestPerStep = 1,
	HarvestStepsCount = 1,
	ResourceValuePerGather = 1,

    QuantityToPlantPerStep = 0,
	PlantingStepsCount = 0,

    CropFieldConfig = "FARM_FIELD_CONFIG_ORCHARD_FARM_APPLE",
    GrowingSetup = "AGENT_WORKSTATION_SETUP_ORCHARD_FARM_GROWING", 
    SowingSetup = "AGENT_WORKSTATION_SETUP_ORCHARD_FARM_SOWING", 
    HarvestSetup = "AGENT_WORKSTATION_SETUP_ORCHARD_FARM_HARVEST"
})



-- Building: Orchard -------------------------------------------------------------------------------------
mod:register({
	DataType = "BUILDING",
	Id = "BUILDING_ORCHARD",
	Name = "BUILDING_ORCHARD_NAME",
	Description = "BUILDING_ORCHARD_DESC",
    BuildingType = "GENERAL",
    AssetCoreBuildingPart = "BUILDING_PART_MONUMENT_POLE", 
    BuildingPartSetList = {
        {
            Name = "BUILDING_PART_SET_ORCHARD1",
            BuildingPartList = { 
                "BUILDING_PART_ORCHARD_FARM1",
                "BUILDING_PART_ORCHARD_CART",
            }
        },
    },	
    RequiredPartList = {
        -- { Category = "CORE", Quantity = 1 }, 
    },
    IsEditable = true,
	VillagerRequired = { Status = "COMMONER", Quantity = 1 }
})



-- Building part: Orchard Farm -------------------------------------------------------------------------------------
mod:register({
	DataType = "BUILDING_PART",
    Id = "BUILDING_PART_ORCHARD_FARM1",
    Name = "BUILDING_PART_ORCHARD_FARM1_NAME",
    Category = "CORE", 
    AssetBuildingFunction = "BUILDING_FUNCTION_ORCHARD",
	BuildingZone = { 
        ZoneEntryList = {
            { 
                Polygon = polygon.createRectangle( {6, 7}, {0.0, 0.0}),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = false } 
            },
            { 
                Polygon = polygon.createRectangle( {5, 6}, {0.0, 0.0}),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } 
            },
		}
	},
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        CoreObjectPrefab = "PREFAB_ORCHARD_FARM1"
    }, 
    ConstructionVisual = "PREFAB_CONSTR_ORCHARD_FARM1", 
    Cost = {
		BuildRightTaxes = {	{ Resource = "GOLD", Quantity = 25 } },
		UpkeepCost = { { Resource = "GOLD", Quantity = 20 } },
		RessourcesNeeded = {
            { Resource = "WOOD", Quantity = 4 },
            { Resource = "PLANK", Quantity = 2 },
		}
    }
})





-- Building Part: Apple Cart -------------------------------------------------------------------------------------
mod:register({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_ORCHARD_CART",
	Name = "BUILDING_PART_ORCHARD_CART_NAME", 
    Category = "MARKET_TABLE",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_ORCHARD_CART"
	},
	BuildingZone = { 
		ZoneEntryList = {
			{ 
				Polygon = polygon.createRectangle( {1, 1}, {0, 0} ),
				Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } 
			}
		}
	},
	AssetBuildingFunction = "BUILDING_FUNCTION_ORCHARD_CART",
	Cost = {
        BuildRightTaxes = {	{ Resource = "GOLD_COINS", Quantity = 1 } },
        UpkeepCost = { { Resource = "GOLD_COINS", Quantity = 1 } },
        RessourcesNeeded = {
            { Resource = "PLANK", Quantity = 2 },
		}
	},
	DesirabilityModifiers = { {
		Desirability = "DESIRABILITY_RESIDENTIAL",
		Type = "ADD",
		Radius = 15.0,
		Value = 0.5
	} } 
})




-- Building Function: Apple Cart -------------------------------------------------------------------------------------
mod:registerEnumValue( "RESOURCE_TYPE", "RESOURCE_TYPE_FRUIT" )
mod:override({ Id = "RESOURCE_FRUIT", ResourceTypeList = { "RESOURCE_FOOD", "FOOD", "RESOURCE_TYPE_FRUIT" } })
mod:register({
    DataType = "BUILDING_FUNCTION_MARKET",
    Id = "BUILDING_FUNCTION_ORCHARD_CART",
	Name = "BUILDING_FUNCTION_ORCHARD_CART_NAME",
    WorkerCapacity = 1,
    RelatedJob = { 
        Job = "MARKET_TENDER", 
        Behavior = "BEHAVIOR_WORK"
    },
	TypeList = { "RESOURCE_TYPE_FRUIT" },
	IsOverrideCapacity = true,
	CapacityPerSlot = 50,
	InputInventoryCapacity = {
		{ Resource = "RESOURCE_FRUIT", Quantity = 50 },
	},
    ResourceListNeeded = {
		{ Resource = "RESOURCE_FRUIT", Quantity = 10 },
	},
	ResourceProduced = {
		{ Resource = "RESOURCE_FRUIT", Quantity = 10 },
	}
})

